@@ -1,4 +1,5 @@
 # Make a structure exporter that works with plone 2.0
+import re
 from csv import writer, QUOTE_MINIMAL
 from xml.dom.minidom import getDOMImplementation
 from ConfigParser import ConfigParser
@@ -31,12 +32,16 @@ from Products.CMFCore.utils import getToolByName
 
 import zLOG
 from config import SEPARATOR, PROPS, ALLPROPS, NONATPROPS, TYPEMAP, DEFAULT_ENCODING
-from config import BRAIN_METHODS
+from config import BRAIN_METHODS, TEXT_GETTERS
 try:
     from App.special_dtml import DTMLFile
 except:
     from Globals import DTMLFile
 
+matchcss = re.compile(r'/portal_css/ploneStyles[\w\-_]*.css')
+matchjs =  re.compile(r'/portal_javascripts/ploneScripts[0-9\-]*.js')
+baseref = re.compile(r'<base href="[\:\w\.\/\-_]*" target="[\w\.\/\-_]*" \/*>')
+
 class ContentExporter(UniqueObject, SimpleItem):
     """An old school Product for plone 2 that sucks out content into plone 3
        generic setup style structure. From Andreas Jung's idea
@@ -73,6 +78,9 @@ class ContentExporter(UniqueObject, SimpleItem):
     security.declareProtected( ManagePortal, 'manage_export' )
     manage_export = PageTemplateFile(os.path.join('www','exportContentMigrator.pt'), globals(),
                                    __name__='manage_export')
+    rooturl = ''
+    portal_css = None
+    portal_javascripts = None
 
     def __init__(self):
         """ Set savepath and log """
@@ -122,6 +130,20 @@ class ContentExporter(UniqueObject, SimpleItem):
                 element = doc[0].createElement(name)
                 element.appendChild(node)
                 doc[1].appendChild(element)
+        elif type(doc) != type(('',)):
+            if listed:
+                if data:
+                    data = SEPARATOR.join(data)
+                else:
+                    data = ''
+            # Retain metadata as tags if its not the page or too long
+            # for inserting into rendered page's head tags        
+            if data and name != 'text':
+                data = str(data)
+                if len(data)<500:
+                    data = data.replace('"', "'")
+                    doc.write(
+                        '<meta name="%s" content="%s" />\n' % (name, data))
         else:
             if listed:
                 if data:
@@ -146,6 +168,8 @@ class ContentExporter(UniqueObject, SimpleItem):
             doc = self.dom.createDocument(None, tag, None)
             node = doc.documentElement
             return (doc, node)
+        elif delimiter == 'HTML':
+            return StringIO()
         else:
             ostream = StringIO()
             return (ostream, writer(ostream,
@@ -170,12 +194,18 @@ class ContentExporter(UniqueObject, SimpleItem):
             else:
                 return output
         else:
+            if headfoot == 'METATAGS':
+                # Get as meta metatags for HTML insert into head
+                return doc.getvalue()
             if headfoot == 'DEFAULT':
                 output = "[%s]\n%s" % (headfoot, doc[0].getvalue())
             elif headfoot:
                 output = "%s\n\n%s" % (doc[0].getvalue(), headfoot)                
             else:
-                output = doc[0].getvalue()
+                try:
+                    output = doc[0].getvalue()
+                except:
+                    return ''
             #if hasattr(os, 'O_BINARY'):
             #    output = output.replace('"\r','')
             #    output = output.replace("\r", '')
@@ -214,14 +244,14 @@ class ContentExporter(UniqueObject, SimpleItem):
         request = getattr(self, 'REQUEST', {})
         if not root:
             root = request.get('root', '')
-        format = request.get('format', 'CSV')
+        fmt = request.get('format', 'CSV')
         if root:
             if not portal:
                 portal = getToolByName(self, 'portal_url').getPortalObject()
             self.export(root=root,
                         portal=portal,
                         users=request.get('exportusers', None),
-                        format = format
+                        format = fmt
                         )
         #if self.dom:
         #    self.dom.unlink()
@@ -488,7 +518,14 @@ class ContentExporter(UniqueObject, SimpleItem):
         """ Based on generic setup folder export to structre -
             See Products.GenericSetup.interfaces.IFilesystemExporter
         """
+        # self.rooturl = '%s%s' % (portal.absolute_url(), root)
+        if getattr(portal, 'changeSkin'):
+            portal.changeSkin('UOBCMS External')
+        self.portal_css = getattr(portal, 'portal_css', None)
+        self.portal_javascripts = getattr(portal, 'portal_javascripts', None)
+        self.rooturl = portal.absolute_url()
         self.format = format
+
         if format == 'XML':
             self.dom = getDOMImplementation()
         self.workflow_tool = getToolByName(portal, 'portal_workflow')
@@ -561,6 +598,26 @@ class ContentExporter(UniqueObject, SimpleItem):
             extra is for extra metadata for the contained objects
             e.g. {meta:'foobar', humbug:'mint'}
         """
+        rendered = False
+        treatas = ''
+        # Some binary objects can be Folderish but they are not Folders - but they have .data raw binary content
+        # however you may have a content folder with a page or a folder called 'data' 
+        # If its a data folder it wil be acquired by its children - so check for folderish property 
+        if hasattr(obj, 'data'):
+            treatas = 'binary'            
+            if hasattr(obj, 'objectIds'):
+                if 'data' in obj.objectIds() or getattr(obj.data, 'isPrincipiaFolderish', 0):
+                    treatas = 'folder'
+        if not treatas:
+            if obj.isPrincipiaFolderish:
+                treatas = 'folder'
+            else:
+                treatas = 'text'
+                if self.format == 'HTML' and getattr(obj, 
+                              'content_type', '') == 'text/html':
+                    doc = self.get_doc('HTML')
+                    rendered = True
+
         modified = self.properties_marshall(obj, doc)
         # do required fields
         for prop, data in PROPS['fixed'].items():
@@ -583,22 +640,37 @@ class ContentExporter(UniqueObject, SimpleItem):
                                data=value,
                                doc=doc)
 
-        if hasattr(obj,'data'):
-            treatas = 'binary'
-        elif obj.isPrincipiaFolderish:
-            treatas = 'folder'
-        else:
-            treatas = 'text'
-
         if treatas == 'text':
-            try:
-                value = obj['text']
-            except:
-                value = getattr(obj, 'text', '')
+            value = ''
+            # Some objs have custom rendering methods to process raw text
+            for get_text in TEXT_GETTERS:
+                getter = getattr(obj, get_text, '')
+                try:
+                    if getter:
+                        value = getter(obj)
+                        break
+                except:
+                    self.out.append('FAILED %s for %s/%s' % (get_text, path, obj.getId()) )
+                    break
+            if not value:
+                try:
+                    value = obj['text']
+                except:
+                    value = getattr(obj, 'text', '')
             if value:
-                if type(value) not in StringTypes:
-                    value = str(value)
-                output = self.get_output(doc, value.replace('\r', ""))
+                if rendered:
+                    # get rendered html page
+                    output = self.render_links(obj, path)
+                    if output:
+                        head = self.get_output(doc, 'METATAGS')
+                        output = output.replace('</head>', '%s\n</head>' % head)
+                else:
+                    if type(value) not in StringTypes:
+                        value = str(value)
+                    output = self.get_output(doc, value.replace('\r', ""))
+            else:
+                output = self.get_output(doc)
+            if output:
                 self.write_file(path, id, output, modified)
             # Handle webpage types which are not isPrincipiaFolderish
             # but are really folders that contain hidden extra items
@@ -721,7 +793,7 @@ class ContentExporter(UniqueObject, SimpleItem):
         """
         modified = None
         for prop, method in PROPS['string'].items():
-            if hasattr(obj,method):
+            if hasattr(obj, method):
                 data = self.stringify(getattr(obj, method)())
             else:
                 data = ''
@@ -761,11 +833,47 @@ class ContentExporter(UniqueObject, SimpleItem):
         if objtype in TYPEMAP.keys():
             propmap = NONATPROPS[objtype]
             for prop in propmap.keys():
-                if hasattr(obj,prop):
+                if hasattr(obj, prop):
                     data = self.stringify(getattr(obj, prop, ''))
                     self.write_element(name=propmap[prop],
                                        data=data, doc=doc)
         return modified
-        
+
+    def render_links(self, obj, path):
+        """ Make links relative for static rendering """
+        try:
+            html = obj.view()
+        except:
+            # Some text/html types may not render directly
+            return str(getattr(obj, 'text', ''))
+        html = baseref.sub('', html) 
+        html = html.replace(self.rooturl, '..')
+        html = html.replace('/%s/' % self.portalname, '../')
+        try:
+            html = self.write_portal_aggregates(html, path)
+        except:
+            pass
+        return html
+
+    def write_portal_aggregates(self, html, path):
+        """ Get aggregated files from portal_css/_javascripts """
+        relative = '../' *  path.count('/')
+        if self.portal_css:
+            matchlist = matchcss.findall(html)
+            for css in matchlist:
+                css_id = css.split('/')[-1]
+                cssfile = str(self.portal_css[css_id])
+                self.write_file('portal_css', css_id, cssfile)
+            html = html.replace('/portal_css', relative + 'portal_css')        
+        if self.portal_javascripts:
+            matchlist = matchjs.findall(html)
+            for js in matchlist:
+                js_id = js.split('/')[-1]
+                jsfile = str(self.portal_javascripts[js_id])
+                self.write_file('portal_javascripts', js_id, jsfile)   
+            html = html.replace('/portal_javascripts', 
+                                relative + 'portal_javascripts')      
+        return html
+
 InitializeClass(ContentExporter)
 
