@@ -59,9 +59,16 @@ NONATPROPS = {'Calendar Item':{'start_date':'startDate',
               'Link':{'remote_url':'remoteUrl'}     
               }
 
-# Extra custom Product's methods for hidden contained objects metadata
-BRAIN_METHODS = {}
-
-#Used to encode content written to text files,
-#when characters are not within the ascii character range
+# Substitute custom Product's methods for hidden contained objects 
+# instead of obj.objectIds()
+BRAIN_METHODS = {'getImageBrains':['Dc_creator', 'Description', 
+                                   'Img_title', 'Img_type'], 
+                 'getFileBrains':['Dc_creator', 'Description']
+                }  
+# Used to encode content written to text files,
+# when characters are not within the ascii character range
 DEFAULT_ENCODING='utf8'
+# Some objects have custom text methods that process it for rendering
+# ... you probably want to export that rather than the raw text so add them here
+# first one that returns the text in the method list is used otherwise obj.text 
+TEXT_GETTERS = ['CookedBody', 'getText']
